"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var os = require("os");
var ts_generator_1 = require("./ts-generator");
var ts_generator_2 = require("./ts-generator");
exports.format = ts_generator_2.format;
function printFieldLikeTypeEmptyCase(field) {
    if (!field.type.isRequired || field.type.name === "ID") {
        return "null";
    }
    if (field.type.isRequired && field.type.isArray && field.type.isScalar) {
        return "[]";
    }
    if (field.type.isRequired &&
        field.type.name === "String" &&
        field.type.isScalar) {
        return "''";
    }
    if (field.type.isRequired &&
        (field.type.name === "Int" || field.type.name === "Float") &&
        field.type.isScalar) {
        return "0";
    }
    if (field.type.isRequired &&
        field.type.name === "Boolean" &&
        field.type.isScalar) {
        return "false";
    }
    if (field.type.isRequired && !field.type.isScalar) {
        return "{ throw new Error('Resolver not implemented') }";
    }
}
function isRootType(name) {
    var rootTypes = ["Query", "Mutation", "Subscription"];
    return rootTypes.indexOf(name) > -1;
}
function generate(args) {
    var files = args.types
        .filter(function (type) { return !isRootType(type.name); })
        .map(function (type) {
        var code = "\n    import { I" + type.name + " } from '[TEMPLATE-INTERFACES-PATH]'\n    import { Types } from './types'\n    " + Array.from(new Set(type.fields
            .filter(function (field) { return !field.type.isEnum && !field.type.isUnion; })
            .filter(function (field) { return !field.type.isScalar; })
            .map(function (field) { return "import { " + field.type.name + "Root } from './" + field.type.name + "'\n  "; }))).join(";") + "\n      " + args.unions
            .filter(function (u) { return type.fields.map(function (f) { return f.type.name; }).indexOf(u.name) > -1; })
            .map(function (u) { return u.types
            .map(function (type) { return "import { " + type.name + "Root } from './" + type.name + "'"; })
            .join(";") + "\n        \n            export type " + u.name + " = " + u.types
            .map(function (type) { return type.name + "Root"; })
            .join("|") + "\n        "; })
            .join(os.EOL) + "\n\n        " + args.enums
            .filter(function (e) { return type.fields.map(function (f) { return f.type.name; }).indexOf(e.name) > -1; })
            .map(function (e) { return "\n        export type " + e.name + " = " + e.values.map(function (v) { return "\"" + v + "\""; }).join("|") + "\n        "; })
            .join(os.EOL) + "\n\n    export interface " + type.name + "Root {\n      " + type.fields
            .map(function (field) { return "\n      " + field.name + (!field.type.isRequired ? "?" : "") + ": " + ts_generator_1.printFieldLikeType(field, false).replace("| null", "") + "\n      "; })
            .join(";") + "\n    }\n\n    export const " + type.name + ": I" + type.name + ".Resolver<Types> = {\n      " + type.fields.map(function (field) { return "\n        " + field.name + ": (root" + (field.arguments.length > 0 ? ", args" : "") + ") => root." + field.name + "\n      "; }) + "\n    }\n    ";
        return {
            path: type.name + ".ts",
            force: false,
            code: code
        };
    });
    files = files.concat(args.types.filter(function (type) { return isRootType(type.name); }).map(function (type) {
        var code = "\n      import { I" + type.name + " } from '[TEMPLATE-INTERFACES-PATH]'\n      import { Types } from './types'\n\n      export interface " + type.name + "Root { }\n      \n      export const " + type.name + ": I" + type.name + ".Resolver<Types> = {\n        " + type.fields.map(function (field) {
            return field.name + ": (root" + (field.arguments.length > 0 ? ", args" : "") + ") => " + printFieldLikeTypeEmptyCase(field);
        }) + "\n      }\n      ";
        return {
            path: type.name + ".ts",
            force: false,
            code: code
        };
    }));
    files.push({
        path: "Context.ts",
        force: false,
        code: "\n    export interface Context {\n      db: any\n      request: any\n    }\n    "
    });
    files.push({
        path: "types.ts",
        force: true,
        code: "\nimport { ITypes } from '[TEMPLATE-INTERFACES-PATH]'\n\n" + args.types
            .map(function (type) { return "import { " + type.name + "Root } from './" + type.name + "'"; })
            .join(";") + "\n\nimport { Context } from './Context'\n\nexport interface Types extends ITypes {\n  Context: Context\n  " + args.types
            .map(function (type) {
            return "" + type.name + (type.type.isEnum || type.type.isUnion ? "" : "Root") + ": " + type.name + (type.type.isEnum || type.type.isUnion ? "" : "Root");
        })
            .join(";") + "\n}\n    "
    });
    files.push({
        path: "index.ts",
        force: false,
        code: "\n    import { IResolvers } from '[TEMPLATE-INTERFACES-PATH]'\n    import { Types } from './types'\n    " + args.types
            .map(function (type) { return "\n      import { " + type.name + " } from './" + type.name + "'\n    "; })
            .join(";") + "\n\n    export const resolvers: IResolvers<Types> = {\n      " + args.types.map(function (type) { return "" + type.name; }).join(",") + "   \n    }\n    "
    });
    return files;
}
exports.generate = generate;
//# sourceMappingURL=ts-scaffolder.js.map